# SPDX-FileCopyrightText: Huawei Inc.
# SPDX-License-Identifier: MIT

# grub's MD-RAID1 implementation uses zero-length array tricks causing
# array-bounds warnings.
# The warnings have been verifited to be harmless in 2.06; for extra
# security, this should be re-checked after updating.
#
# Removing -Werror=array-bounds here allows us to use -Werror=array-bounds
# globally in OPTIMIZE_FOR=security mode.

TARGET_CFLAGS:remove = "-Werror=array-bounds"

# Make sure that GRUB can load squashfs, so that it can mount the system image
# and load the kernel, or other modules, from there directly. In addition add
# echo and halt modules, so that the GRUB boot script can print diagnostic
# messages or shut down on error.
GRUB_BUILDIN:append = " squash4 halt echo"

# TODO oe-core avoids armv7ve builds. This is more of a blanket fix because
# grub supports this arch. It doesn't support hardfp configuration that can't
# be forced into softfp with a compiler flag. qemuarm-efi defaults to softfp.
COMPATIBLE_HOST:qemuarm-efi = 'arm.*-(linux.*|freebsd.*)'

# depending on oniro-grub-bootconf:do_deploy in order to obtaing grub.cfg 
do_mkimage_grub-efi[depends] += "oniro-grub-bootconf:do_deploy"
inherit deploy
do_mkimage:append() {
	cd ${B}
	GRUB_MKIMAGE_MODULES="${GRUB_BUILDIN}"

	# If 'all' is included in GRUB_BUILDIN we will include all available grub2 modules
	if [ "${@ bb.utils.contains('GRUB_BUILDIN', 'all', 'True', 'False', d)}" = "True" ]; then
		bbdebug 1 "Including all available modules"
		# Get the list of all .mod files in grub-core build directory
		GRUB_MKIMAGE_MODULES=$(find ${B}/grub-core/ -type f -name "*.mod" -exec basename {} .mod \;)
	fi
    # embedding the final grub.cfg that was installed by oniro-grub-bootconf into deploydir
	grub-mkimage -v -c ${DEPLOY_DIR}/images/${MACHINE}/grub.cfg -p ${EFIDIR} -d ./grub-core/ \
	               -O ${GRUB_TARGET}-efi -o ./${GRUB_IMAGE_PREFIX}${GRUB_IMAGE} \
	               ${GRUB_MKIMAGE_MODULES}
}