# SPDX-FileCopyrightText: Huawei Inc.
#
# SPDX-License-Identifier: MIT

SUMMARY = "GRUB configuration file specialized for Oniro reference images"
DESCRIPTION = "This GRUB configuration file has the following features: \
- Support for A/B immutable system image \
- Support for try-mode booting compatible with RAUC \
- Kernel is loaded from the system image \
"
LICENSE = "Apache-2.0"
LIC_FILES_CHKSUM = "file://${COMMON_LICENSE_DIR}/Apache-2.0;md5=89aea4e17d99a7cacdbeed46a0096b10"

SRC_URI = " \
	file://grub.cfg \
	file://grubenv \
"
require recipes-core/rauc/rauc-conf.inc
inherit deploy

RPROVIDES:${PN} += "virtual-grub-bootconf"

WIC_BOOTLOADER_TIMEOUT ?= "0"

# Get definitions of the EFI_ variables.
require conf/image-uefi.conf

S = "${WORKDIR}"

do_install() {
	# Install the boot assets into the rootfs. Those can be used to handle boot
	# asset updates by SysOTA later on.
	install -d ${D}${EFI_FILES_PATH}
	install -m 644 grubenv ${D}${EFI_FILES_PATH}/grubenv
}

do_deploy() {
	#expanding the variables..
	sed -i "s/@KERNEL_IMAGETYPE@/${KERNEL_IMAGETYPE}/g" ${WORKDIR}/grub.cfg
	baudrate=`echo "${SERIAL_CONSOLES}" | sed 's/\;.*//'`
	ttydev=`echo "${SERIAL_CONSOLES}" | sed -e 's/^[0-9]*\;//' -e 's/ .*//'`
	sed -i "s/@SERIAL_CONSOLES@/${ttydev},${baudrate}/g" ${WORKDIR}/grub.cfg
	sed -i "s,@RAUC_SLOT_A@,${RAUC_SLOT_A},g" ${WORKDIR}/grub.cfg
	sed -i "s,@RAUC_SLOT_B@,${RAUC_SLOT_B},g" ${WORKDIR}/grub.cfg
	sed -i "s,@WIC_BOOTLOADER_TIMEOUT@,${WIC_BOOTLOADER_TIMEOUT},g" ${WORKDIR}/grub.cfg
	# Install the boot assets into DEPLOYDIR. The deploy bbclass
	# eventually copies those into the boot partition.
	install -m 644 ${WORKDIR}/grub.cfg ${DEPLOYDIR}
	# Unlike grub.cfg, nothing installs this file to the boot partition
	# automatically. It is handled by extending IMAGE_EFI_BOOT_FILES from
	# conf/distro/oniro-linux.conf.
	install -m 644 ${D}${EFI_FILES_PATH}/grubenv ${DEPLOYDIR}
}

# Cargo-cult from a similar recipe.
addtask deploy after do_install before do_build

FILES:${PN} = "\
	${EFI_FILES_PATH}/grub.cfg \
	${EFI_FILES_PATH}/grubenv \
"

PACKAGE_ARCH = "${MACHINE_ARCH}"
